SpellShader = {}

BUFFER_TIME = 1500 -- milliseconds, extra time added for applying less important (de)buffs 
--[[
The spell shader can apply custom effects to the Spell object. The spell object 
contains the texture and font objects, so it's easy to change them based on your
own preferences. 

There is a generic SpellShader effect which is applied to all spells which do not have 
a custom spell shader. To create a custom spell shader for a spell, simply create a 
function in the format SpellShader:_<SPELL_ID>. For example, if I wanted to apply different
heuristics to Bane of Doom, I would create this function

-- 603 is the id of Bane of Doom
function SpellShader:_603()
	self.spell.texture:SetVertexColor(0,1,0); --apply a green mask over the texture.
end

For special occasions, you can also create _<SPELL_ID>_init() function which gets executed on
object creation for your spell.

]]--
function SpellShader:new(spell)
	local object = { spell = spell, doShade = nil }
	local funcname = "_";
	local funcname_init = nil;
	if spell.id then
		funcname = funcname .. spell.id;
		funcname_init = funcname .. "_init";
	end

	setmetatable (object, { __index = SpellShader } )
	
	if self[funcname] == nil then
		object.doShade = self["generic"];
	else
		object.doShade = self[funcname];
	end
	
	if self[funcname_init] ~= nil then
		self[funcname_init](object);
	end
	return object
end

--taste for blood
function SpellShader:_56638()
	if self.spell.timeleft > 4000 then
		self.spell:updateDuration((self.spell.timeleft - 4000) / 1000)
	else
		self.spell:updateDuration((self.spell.timeleft) / 1000)
	end 
end

--Disable faerie fire stacks
function SpellShader:_770_init()
	self.spell.disableStacks = true;
end

--Disable stampede stacks
function SpellShader:_78893_init()	
	self.spell.disableStacks = true;
end

--Disable Predator's Swiftness stacks
function SpellShader:_69369_init()
	self.spell.disableStacks = true;
end

--Gouge correct for stacks
function SpellShader:_1776_init()
	self.spell.disableStacks = true;
end

--Revealing Strike correct for stacks
function SpellShader:_84617_init()
	self.spell.disableStacks = true;
end

--Master of Shadows correct for stacks
function SpellShader:_31223_init()
	self.spell.disableStacks = true;
end

--curse of gul'dan
function SpellShader:_86000()
	local start, duration, enabled = GetSpellCooldown(self.spell.props.applied_by_id)
	
	if duration == 12 then
		duration = duration - (GetTime() - start)
		self.spell:updateDuration(duration);
		self.spell.texture:SetAlpha(SPELL_OPACITY);
		self.spell.texture:SetDesaturated(true);
	else		
		self.spell.timefont:SetText("");
		self.spell.texture:SetDesaturated(false);
		self.spell.texture:SetAlpha(1);
	end
end

--function SpellShader:_603()
--	print(	"bane of doom" ) 
--end

function SpellShader:generic()
	local cast_threshold = 0;
	if self.spell.props.spell_type == DOT then
		local down, up, lagHome, lagWorld = GetNetStats();
		-- patch 6.0.2 allows refreshing dots at the 30% mark of the duration
		cast_threshold = self.spell.totalDuration * 0.3 + self.spell.props.castTime + lagWorld;	
		if self.spell.timeleft <= self.spell.props.castTime + lagWorld then
			--dot will drop off			
			self.spell.texture:SetVertexColor(1,0,0);
			self.spell.texture:SetDesaturated(false);
			self.spell.texture:SetAlpha(1);
		elseif self.spell.timeleft <= cast_threshold then	
			--optimum time to cast gogogo
			self.spell.texture:SetDesaturated(false);
			self.spell.texture:SetAlpha(1);
			self.spell.texture:SetVertexColor(1,1,1);
		else
			--it will be a while before it drops off
			self.spell.texture:SetVertexColor(1,1,1);
			self.spell.texture:SetAlpha(SPELL_OPACITY);
			self.spell.texture:SetDesaturated(true);
		end
	elseif self.spell.props.spell_type == DEBUFF or self.spell.props.spell_type == BUFF then
		if self.spell.props.castTime then
			local down, up, lagHome, lagWorld = GetNetStats();
			cast_threshold = self.spell.props.castTime + lagWorld + BUFFER_TIME;	
			if self.spell.timeleft <= cast_threshold then			
				--optimum time to cast gogogo
				self.spell.texture:SetDesaturated(false);
				self.spell.texture:SetAlpha(1);
				self.spell.texture:SetVertexColor(1,1,1);
			else
				--it will be a while before it drops off
				self.spell.texture:SetVertexColor(1,1,1);
				self.spell.texture:SetAlpha(SPELL_OPACITY);
				self.spell.texture:SetDesaturated(true);
			end
		end
	end
end

